From: Andreas Bolka
Date: Mon, 14 Dec 2009 01:49:39 +0100
Subject: [PATCH] Implement shared library loading for POSIX

This makes extensions work on Linux:

  >> import %ext/sample.so
  >> add-mul 10 20 30
  == 900

---
 makefile                |    2 +-
 src/os/posix/host-lib.c |   15 +++++++--------
 2 files changed, 8 insertions(+), 9 deletions(-)

diff --git a/makefile b/makefile
index 26bb501..1f80fac 100644
--- a/makefile
+++ b/makefile
@@ -2,7 +2,7 @@
 
 C= gcc 
 CFLAGS= -c -O1 -fpack-struct -D_FILE_OFFSET_BITS=64 -I src/include/
-LIBS= libr3.so
+LIBS= libr3.so -ldl
 
 #-lc -lm -lstdc++
 
diff --git a/src/os/posix/host-lib.c b/src/os/posix/host-lib.c
index a8d5e1c..4144fbb 100644
--- a/src/os/posix/host-lib.c
+++ b/src/os/posix/host-lib.c
@@ -41,6 +41,7 @@
 #include <sys/types.h>
 #include <time.h>
 #include <string.h>
+#include <dlfcn.h>
 
 #ifndef timeval // for older systems
 #include <sys/time.h>
@@ -426,10 +427,9 @@ static void *Task_Ready;
 **
 ***********************************************************************/
 {
-	void *dll;
-
-	//= LoadLibrary(path);
-	//*error = GetLastError();
+	void *dll = dlopen(path, RTLD_LAZY/*|RTLD_GLOBAL*/);
+	*error = 0; // dlerror() returns a char* error message, so there's
+				// no immediate way to return an "error code" in *error
 
 	return dll;
 }
@@ -443,7 +443,7 @@ static void *Task_Ready;
 **
 ***********************************************************************/
 {
-	//FreeLibrary((HINSTANCE)dll);
+	dlclose(dll);
 }
 
 
@@ -455,10 +455,9 @@ static void *Task_Ready;
 **
 ***********************************************************************/
 {
-	void *fp; // = GetProcAddress((HMODULE)dll, funcname);
-	//DWORD err = GetLastError();
+	void *fp = dlsym(dll, funcname);
 
-	return 0;
+	return fp;
 }
 
 
